using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using FitConnect;
using FitConnect.Models;
using FluentAssertions;
using Newtonsoft.Json;
using NUnit.Framework;

namespace IntegrationTests.Sender;

[TestFixture]
public class SenderTestHappyPath : SenderTestBase {
    [Test]
    public void CheckIfSecretsAreValid() {
        ClientId.Should().NotBe("00000000-0000-0000-0000-000000000000");
        ClientSecret.Should().NotBe("0000000000000000000000000000000000000000000");
    }


    [TestCase("0b8e6fbd-62e2-4b6f-b333-5308d82e0a00")]
    [TestCase("d2be2027-9368-4c0c-a265-2fdbf7ecd4d9")]
    public void GetSubmissionStatus(string caseId) {
        var status =
            Sender.GetEventLogAsync(Guid.Parse(caseId),
                    new Guid("aa3704d6-8bd7-4d40-a8af-501851f93934"), true)
                .Result;
        status.ForEach(s => Console.WriteLine($"{s.EventTime} - {s.EventType}"));
        status.Count.Should().BeGreaterThan(0);
    }

    [Test]
    public void Submit_FinishSubmission_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var attachment = Attachment.FromPath("Test.pdf",
            System.Net.Mime.MediaTypeNames.Application.Pdf, description: "Just an attachment",
            fileName: null);
        attachments.Add(attachment);
        var dut = SendableSubmission.Builder()
            .SetDestination(DestinationId)
            .SetServiceType("urn:de:fim:leika:leistung:99400048079000", "ServiceName")
            .SetJsonData(@"{""data"":""value""}",
                new Uri(JSON_SCHEMA_URI))            
            .AddAttachments(attachments.ToArray())
            .Build();

        // Act
        var submission = Sender.SendAsync(dut).Result;

        // Assert
        submission.Should().NotBeNull();
        Console.WriteLine($"Case ID: {submission.CaseId}");
    }

    [Test]
    public void Submit_FinishSubmissionWithData_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var content = File.ReadAllBytes("Test.pdf");
        var attachment = Attachment.FromByteArray(content, "application/pdf", "RandomBytes.pdf",
            "Just a test");

        attachments.Add(attachment);
        var dut = SendableSubmission.Builder()
            .SetDestination(DestinationId)
            .SetServiceType("urn:de:fim:leika:leistung:99400048079000", "ServiceName")
            .SetJsonData(JsonConvert.SerializeObject(new {
                FirstName = "John",
                LastName = "Doe",
                Age = 42,
                Birthday = DateTime.Today.AddYears(-42)
            }), new Uri(JSON_SCHEMA_URI))
           
            .AddAttachments(attachments)
            .Build();

        // Act
        var submission = Sender.SendAsync(dut).Result;

        // Assert
        var statusForSubmission =
            Sender.GetEventLogAsync(submission.CaseId, DestinationId).Result;


        statusForSubmission.Should().HaveCountGreaterThan(0);
        Console.WriteLine($"Case ID: {submission.CaseId}");
        Console.WriteLine($"Submission ID: {submission.SubmissionId}");
        submission.Should().NotBeNull();
    }

    [Ignore("Destination can not be found")]
    [Test]
    public void Submit_FinishSubmissionWithDataFindDestination_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var content = File.ReadAllBytes("Test.pdf");
        var attachment = Attachment.FromByteArray(content, "application/pdf", "RandomBytes.pdf",
            "Just a test");

        attachments.Add(attachment);

        var route = ClientFactory.GetRoutingClient(FitConnectEnvironment.Testing)
            .FindDestinationsAsync("99400048079000", ars: "09372126").Result.First();

        var dut = SendableSubmission.Builder()
            .SetDestination(route.DestinationId)
            .SetServiceType("urn:de:fim:leika:leistung:99400048079000", "ServiceName")
            .SetJsonData(JsonConvert.SerializeObject(new {
                FirstName = "John",
                LastName = "Doe",
                Age = 42,
                Birthday = DateTime.Today.AddYears(-42)
            }), new Uri(JSON_SCHEMA_URI))
            .AddAttachments(attachments)
            .Build();

        // Act
        var submission = Sender.SendAsync(dut).Result;

        // Assert
        var statusForSubmission =
            Sender.GetEventLogAsync(submission.SubmissionId, DestinationId).Result;

        statusForSubmission.Should().HaveCountGreaterThan(0);
        Console.WriteLine($"Case ID: {submission.CaseId}");
        Console.WriteLine($"Submission ID: {submission.SubmissionId}");
        submission.Should().NotBeNull();
    }

    [Test]
    public void SubmissionBuilder_AnnounceJsonAddXml_ShouldFail() {
        Assert.Throws<ArgumentException>(() => {
            SendableSubmission.Builder().SetDestination(Guid.NewGuid())
                .SetServiceType(ServiceIdentifier, "")
                .SetJsonData(@"<user><name>Max</name><age>40</age></user>", new Uri(XML_SCHEMA_URI))
                .AddAttachments()
                .Build();
        });
    }

    [Test]
    public void SubmissionBuilder_AnnounceXmlAddXml_ShouldPass() {
        SendableSubmission.Builder().SetDestination(Guid.NewGuid())
            .SetServiceType(ServiceIdentifier, "")
            .SetXmlData(@"<user><name>Max</name><age>40</age></user>", new Uri(XML_SCHEMA_URI))
            .AddAttachments()
            .Build();
    }

    [Test]
    public void SubmissionBuilder_AnnounceXmlAddJson_ShouldFail() {
        Assert.Throws<ArgumentException>(() => {
            SendableSubmission.Builder().SetDestination(Guid.NewGuid())
                .SetServiceType(ServiceIdentifier, "")
                .SetXmlData(@"{""name"":""Max"",""age"":40}", new Uri(JSON_SCHEMA_URI))
                .AddAttachments()
                .Build();
        });
    }

    [Test]
    public void SubmissionBuilder_AnnounceJsonAddJson_ShouldPass() {
        SendableSubmission.Builder().SetDestination(Guid.NewGuid())
            .SetServiceType(ServiceIdentifier, "")
            .SetJsonData(@"{""name"":""Max"",""age"":40}", new Uri(JSON_SCHEMA_URI))
            .AddAttachments()
            .Build();
    }
}
